import * as mongodb from 'mongodb';
import { IMongoDefinition } from './types/types';

let client: mongodb.MongoClient;
let collection: mongodb.Collection;

const MongoClient = mongodb.MongoClient;

/**
 * @description It keeps MongoDb connection in RAM.
 *
 * @export
 * @param {IMongoDefinition} config  Configuration to mongo
 * @param {((
 *         err: Error | undefined,
 *         collection: mongodb.Collection | undefined
 *     ) => void)} cb Callback function with error and db.
 */
export default function getMongoDb(
    config: IMongoDefinition,
    cb: (
        err: Error | undefined,
        collection: mongodb.Collection | undefined,
    ) => void,
): void {
    if (client) {
        cb(undefined, collection);
    } else {
        getMongoDbPromise(config).then((resp) => cb(undefined, resp)).catch((err) => cb(err, undefined));
    }
}

/**
 * @description It keeps MongoDb connection in RAM.
 *
 * @export
 * @param {IMongoDefinition} config Configuration to mongo
 * @return {*}  {Promise<mongodb.Collection>}
 */
export async function getMongoDbPromise(config: IMongoDefinition) : Promise<mongodb.Collection>{
    if (client) {
        return collection;
    } else {
        try {
            const newClient = await MongoClient.connect(
                config.url,
                config.options,
            );
            client = newClient;
            const db = client.db(config.database);
            collection = db.collection(config.collection);
            return  collection;
        } catch (err) {
            const error = err as Error;
            const logErr = {
                error: error,
                extraInfo: {
                    function: `${__filename}[${getMongoDb.name}]`,
                    config: config,
                },
            };

            console.error(logErr);

            throw error;
        }
    }
}

/**
 * @description Close MongoDb connection
 * @export
 * @param {() => void} cb
 */
//@ts-ignore
/*export function closeMongoDb(cb: () => void): void {
    closeMongoDbPromise().then(() => cb());
}
*/
/**
 * @description Close MongoDb connection
 * @export
 */
//@ts-ignore
/*export async function closeMongoDbPromise(): Promise<void> {
    if (client) {
        await client.close();
    }
}*/